==> pendo_sdk-3.0.0.tar.gz: lib/src/pendo_navigation_observer.dart <==
/*
 *  * Copyright © 2019 Pendo.io. All rights reserved.
 */

import 'package:flutter/material.dart';
// ignore: prefer_relative_imports
import 'package:pendo_sdk/src/logger.dart';
import 'mixins/screenid_parser.dart';

/// Typedef for a route validation function.
///
/// A [RouteValidator] is a function that takes a [Route<dynamic>] as input and
/// returns a [bool] indicating whether the route is considered valid.
/// by default any [ModalRoute] is tracked
typedef RouteValidator = bool Function(Route<dynamic> route);
bool defaultRouteValidator(Route<dynamic> route) => route is ModalRoute;

/// Typedef for a screen name extraction function.
///
/// A [RouteNameExtractor] is a function that takes a [Route<dynamic>] as input
/// and returns a [String] or [null] representing the screen name to be used
/// for analytics or tracking purposes. It is used to extract human-readable
/// screen names from route settings or an identifier for anonymous routes.
typedef RouteNameExtractor = String? Function(Route<dynamic> route);
String? defaultRouteNameExtractor(Route<dynamic> route) => route.settings.name;

class PendoNavigationObserver extends RouteObserver<ModalRoute<dynamic>> with ScreenIdParser {
  static const String FILE_TAG = 'PendoNavigationObserver:';
  static String? _routeName;
  static String? get routeName => _routeName;

  RouteValidator isRouteValid;
  RouteNameExtractor routeNameExtractor;
  PendoNavigationObserver({this.isRouteValid = defaultRouteValidator, this.routeNameExtractor = defaultRouteNameExtractor});

  @override
  void didPush(Route<dynamic> route, Route<dynamic>? previousRoute) {
    Logger.i('$FILE_TAG didPush');
    super.didPush(route, previousRoute);
    _extractRouteName(route);
  }

  @override
  void didReplace({Route<dynamic>? newRoute, Route<dynamic>? oldRoute}) {
    Logger.i('$FILE_TAG didReplace');
    super.didReplace(newRoute: newRoute, oldRoute: oldRoute);

    if (newRoute != null) {
      _extractRouteName(newRoute);
    }
  }

  @override
  void didPop(Route<dynamic> route, Route<dynamic>? previousRoute) {
    Logger.i('$FILE_TAG didPop');
    super.didPop(route, previousRoute);

    if (previousRoute != null) {
      _extractRouteName(previousRoute);
    }
  }

  void _extractRouteName(Route<dynamic> route) {
    if (!isRouteValid(route)) {
      Logger.e('$FILE_TAG Route should not be empty');
      return;
    }

    // 1. extracting route name from settings name
    String? routeName = routeNameExtractor(route);

    // 2. extracting route name from Page key
    if (routeName == null) {
      routeName = pageKeyExtractionFromRoute(route);
    }

    // 3. extracting route name from builder
    if (routeName == null) {
      routeName = extractRouteNameFromRouteTypes(route);
    }

    // 4. route is root
    if (routeName == '/') {
      routeName = 'flutteroot';
    }

    Logger.i('$FILE_TAG extractRouteName: $routeName');
    _routeName = routeName;
  }
}

==> pendo_sdk-3.2.0.tar.gz: lib/src/pendo_navigation_observer.dart <==
/*
 *  * Copyright © 2019 Pendo.io. All rights reserved.
 */

import 'package:flutter/material.dart';
// !! we ignore using relative imports only for known issue with logger.dart static fields
// ignore: prefer_relative_imports
import 'package:pendo_sdk/src/logger.dart';
import 'mixins/screenid_parser.dart';
import 'utils/common_utils.dart';

/// Typedef for a route validation function.
///
/// A [RouteValidator] is a function that takes a [Route<dynamic>] as input and
/// returns a [bool] indicating whether the route is considered valid.
/// by default any [ModalRoute] is tracked
typedef RouteValidator = bool Function(Route<dynamic> route);
bool defaultRouteValidator(Route<dynamic> route) => route is ModalRoute;

/// Typedef for a screen name extraction function.
///
/// A [RouteNameExtractor] is a function that takes a [Route<dynamic>] as input
/// and returns a [String] or [null] representing the screen name to be used
/// for analytics or tracking purposes. It is used to extract human-readable
/// screen names from route settings or an identifier for anonymous routes.
typedef RouteNameExtractor = String? Function(Route<dynamic> route);
String? defaultRouteNameExtractor(Route<dynamic> route) => route.settings.name;

const String DEFAULT_SCREEN_NAME = 'DefaultScreenId';
const String ROOT_SCREEN_NAME = 'flutteroot';

class PendoNavigationObserver extends RouteObserver<ModalRoute<dynamic>> with ScreenIdParser {
  static const String FILE_TAG = 'PendoNavigationObserver:';
  static String _currentRouteName = DEFAULT_SCREEN_NAME;
  static String _previousRouteName = DEFAULT_SCREEN_NAME;
  static String get currentRouteName => _currentRouteName;
  static String get previousRouteName => _previousRouteName;

  RouteValidator isRouteValid;
  RouteNameExtractor routeNameExtractor;
  PendoNavigationObserver({this.isRouteValid = defaultRouteValidator, this.routeNameExtractor = defaultRouteNameExtractor});

  @override
  void didPush(Route<dynamic> route, Route<dynamic>? previousRoute) {
    Logger.i('$FILE_TAG didPush');
    super.didPush(route, previousRoute);
    _extractRouteName(route);
  }

  @override
  void didReplace({Route<dynamic>? newRoute, Route<dynamic>? oldRoute}) {
    Logger.i('$FILE_TAG didReplace');
    super.didReplace(newRoute: newRoute, oldRoute: oldRoute);

    if (newRoute != null) {
      _extractRouteName(newRoute);
    }
  }

  @override
  void didPop(Route<dynamic> route, Route<dynamic>? previousRoute) {
    Logger.i('$FILE_TAG didPop');
    super.didPop(route, previousRoute);

    if (previousRoute != null) {
      _extractRouteName(previousRoute);
    }
  }

  void _extractRouteName(Route<dynamic> route) {
    if (!isRouteValid(route)) {
      Logger.e('$FILE_TAG Route should not be empty');
      return;
    }

    // 1. extracting route name from settings name
    String? routeName = routeNameExtractor(route);

    // 2. extracting route name from Page key
    if (CommonUtils.isNullOrEmptyString(routeName)) {
      routeName = pageKeyExtractionFromRoute(route);
    }

    // 3. extracting route name from builder
    if (CommonUtils.isNullOrEmptyString(routeName)) {
      routeName = extractRouteNameFromRouteTypes(route);
    }

    // 4. route is root
    if (routeName == '/') {
      routeName = ROOT_SCREEN_NAME;
    }

    if (CommonUtils.isNullOrEmptyString(routeName)) {
      Logger.w('$FILE_TAG extractRouteName is empty, setting default name. route: $route');
      routeName = DEFAULT_SCREEN_NAME;
    } else {
      Logger.i('$FILE_TAG extractRouteName: $routeName');
    }

    _previousRouteName = _currentRouteName;
    _currentRouteName = routeName!;
  }
}
